/* eslint-disable unicorn/filename-case */
import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { DateTime } from "luxon";
import { OffsetDirection } from "../../types/OffsetDirection.js";
import { formatOffset } from "../../util/formatOffset.js";
import { isValidDate } from "../../util/isValidDate.js";
import { isValidDateTime } from "../../util/isValidDateTime.js";
import { pad } from "../../util/pad.js";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { throwPgTPError } from "../../util/throwPgTPError.js";
import { Time } from "./Time.js";
import { TimestampTZ } from "./TimestampTZ.js";
class TimeTZConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        const [argument, ...otherArguments] = context.data, allowedTypes = [ParsedType.number, ParsedType.string, ParsedType.object, ParsedType["globalThis.Date"], ParsedType["luxon.DateTime"]], parsedType = getParsedType(argument);
        if (parsedType !== ParsedType.number && context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                });
            return INVALID;
        }
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case "string":
                return this._parseString(context, argument);
            case "number":
                return this._parseNumber(context, argument, otherArguments);
            default:
                return this._parseObject(context, argument);
        }
    }
    _parseString(context, argument) {
        const parsedTimestamp = TimestampTZ.safeFrom(argument);
        if (parsedTimestamp.success)
            return OK(new TimeTZClass(parsedTimestamp.data.hour, parsedTimestamp.data.minute, parsedTimestamp.data.second, parsedTimestamp.data.offset));
        this.setIssueForContext(context, {
            code: "invalid_string",
            received: argument,
            expected: "LIKE HH:MM:SS+HH:MM",
        });
        return INVALID;
    }
    _parseNumber(context, argument, otherArguments) {
        const totalLength = otherArguments.length + 1;
        if (totalLength === 1) {
            return this._parseObject(context, DateTime.fromMillis(argument, {
                zone: "UTC",
            }));
        }
        if (totalLength !== 6) {
            this.setIssueForContext(context, totalLength > 6
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 6,
                    exact: true,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 6,
                    exact: true,
                });
            return INVALID;
        }
        // Should be [hour, minute, second, offsetHour, offsetMinute, offsetDirection]
        // To validate all the arguments, we make them go through the object parser
        return this._parseObject(context, {
            hour: argument,
            minute: otherArguments[0],
            second: otherArguments[1],
            offset: {
                hour: otherArguments[2],
                minute: otherArguments[3],
                direction: otherArguments[4],
            },
        });
    }
    _parseObject(context, argument) {
        // Should be [TimeTZ]
        if (TimeTZ.isTimeTZ(argument))
            return OK(new TimeTZClass(argument.hour, argument.minute, argument.second, argument.offset));
        // Should be [globalThis.Date]
        const jsDate = isValidDate(argument);
        if (jsDate.isOfSameType) {
            if (jsDate.isValid) {
                const [hour, minute, second, offset] = [jsDate.data.getHours(), jsDate.data.getMinutes(), jsDate.data.getSeconds(), jsDate.data.getTimezoneOffset()];
                return OK(new TimeTZClass(hour, minute, second, {
                    hour: offset / 60,
                    minute: offset % 60,
                    /* c8 ignore next 2 */
                    // globalThis.Date.getTimezoneOffset() returns system timezone offset, so it's always positive or negative depending on the system timezone
                    direction: offset < 0 ? OffsetDirection.minus : OffsetDirection.plus,
                }));
            }
            this.setIssueForContext(context, jsDate.error);
            return INVALID;
        }
        // Should be [luxon.DateTime]
        const luxonDate = isValidDateTime(argument);
        if (luxonDate.isOfSameType) {
            if (luxonDate.isValid) {
                const [hour, minute, second, offset] = [luxonDate.data.hour, luxonDate.data.minute, luxonDate.data.second, luxonDate.data.offset];
                return OK(new TimeTZClass(hour, minute, second, {
                    hour: offset / 60,
                    minute: offset % 60,
                    direction: offset < 0 ? OffsetDirection.minus : OffsetDirection.plus,
                }));
            }
            this.setIssueForContext(context, luxonDate.error);
            return INVALID;
        }
        // Should be [TimeTZObject]
        const parsedObject = hasKeys(argument, [
            ["hour", "number"],
            ["minute", "number"],
            ["second", "number"],
            ["offset", "object"],
        ]);
        if (!parsedObject.success) {
            switch (true) {
                case parsedObject.otherKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "unrecognized_keys",
                        keys: parsedObject.otherKeys,
                    });
                    break;
                case parsedObject.missingKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "missing_keys",
                        keys: parsedObject.missingKeys,
                    });
                    break;
                case parsedObject.invalidKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "invalid_key_type",
                        ...parsedObject.invalidKeys[0],
                    });
                    break;
            }
            return INVALID;
        }
        const { hour, minute, second, offset } = parsedObject.obj, parsedOffset = hasKeys(offset, [
            ["hour", "number"],
            ["minute", "number"],
            ["direction", "string"],
        ]);
        if (!parsedOffset.success) {
            switch (true) {
                case parsedOffset.otherKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "unrecognized_keys",
                        keys: parsedOffset.otherKeys,
                    });
                    break;
                case parsedOffset.missingKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "missing_keys",
                        keys: parsedOffset.missingKeys,
                    });
                    break;
                case parsedOffset.invalidKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "invalid_key_type",
                        ...parsedOffset.invalidKeys[0],
                    });
                    break;
            }
            return INVALID;
        }
        if (hour % 1 !== 0) {
            this.setIssueForContext(context, {
                code: "not_whole",
            });
            return INVALID;
        }
        if (hour < 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
            });
            return INVALID;
        }
        if (hour > 23) {
            this.setIssueForContext(context, {
                code: "too_big",
                maximum: 23,
                type: "number",
                inclusive: true,
            });
            return INVALID;
        }
        if (minute % 1 !== 0) {
            this.setIssueForContext(context, {
                code: "not_whole",
            });
            return INVALID;
        }
        if (minute < 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
            });
            return INVALID;
        }
        if (minute > 59) {
            this.setIssueForContext(context, {
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
            });
            return INVALID;
        }
        if (second < 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
            });
            return INVALID;
        }
        if (second >= 60) {
            this.setIssueForContext(context, {
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
            });
            return INVALID;
        }
        // Validate the offset
        if (parsedOffset.obj.direction !== OffsetDirection.minus && parsedOffset.obj.direction !== OffsetDirection.plus) {
            this.setIssueForContext(context, {
                code: "invalid_string",
                expected: [OffsetDirection.minus, OffsetDirection.plus],
                received: parsedOffset.obj.direction,
            });
            return INVALID;
        }
        if (parsedOffset.obj.hour % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
            });
        }
        if (parsedOffset.obj.hour < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
            });
        }
        if (parsedOffset.obj.hour > 23) {
            throwPgTPError({
                code: "too_big",
                maximum: 23,
                type: "number",
                inclusive: true,
            });
        }
        if (parsedOffset.obj.minute % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
            });
        }
        if (parsedOffset.obj.minute < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
            });
        }
        if (parsedOffset.obj.minute > 59) {
            throwPgTPError({
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
            });
        }
        return OK(new TimeTZClass(hour, minute, second, offset));
    }
    isTimeTZ(object) {
        return object instanceof TimeTZClass;
    }
}
const TimeTZ = new TimeTZConstructorClass();
class TimeTZClass extends PgTPBase {
    constructor(_hour, _minute, _second, _offset) {
        super();
        this._hour = _hour;
        this._minute = _minute;
        this._second = _second;
        this._offset = _offset;
    }
    _equals(context) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = TimeTZ.safeFrom(...context.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(context, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return `${pad(this._hour)}:${pad(this._minute)}:${pad(this._second)}${formatOffset(this._offset, { returnEmpty: true })}`;
    }
    toNumber() {
        const todayUnix = DateTime.now().setZone("UTC").startOf("day").toMillis();
        return this.toDateTime("UTC").toMillis() - todayUnix;
    }
    toJSON() {
        return {
            hour: this._hour,
            minute: this._minute,
            second: this._second,
            offset: this._offset,
        };
    }
    toTime() {
        return Time.from({
            hour: this._hour,
            minute: this._minute,
            second: this._second,
        });
    }
    toDateTime(zone) {
        const today = DateTime.now();
        return DateTime.fromISO(`${pad(today.year, 4)}-${pad(today.month)}-${pad(today.day)}T${pad(this._hour)}:${pad(this._minute)}:${pad(this._second)}${formatOffset(this._offset, {
            returnZ: true,
        })}`, { setZone: true }).setZone(zone);
    }
    toJSDate(zone) {
        return this.toDateTime(zone).toJSDate();
    }
    get hour() {
        return this._hour;
    }
    set hour(hour) {
        const parsedType = getParsedType(hour);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (hour % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
            });
        }
        if (hour < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
            });
        }
        if (hour > 23) {
            throwPgTPError({
                code: "too_big",
                maximum: 23,
                type: "number",
                inclusive: true,
            });
        }
        this._hour = hour;
    }
    get minute() {
        return this._minute;
    }
    set minute(minute) {
        const parsedType = getParsedType(minute);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (minute % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
            });
        }
        if (minute < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
            });
        }
        if (minute > 59) {
            throwPgTPError({
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
            });
        }
        this._minute = minute;
    }
    get second() {
        return this._second;
    }
    set second(second) {
        const parsedType = getParsedType(second);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (second < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
            });
        }
        if (second >= 60) {
            throwPgTPError({
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
            });
        }
        this._second = second;
    }
    get offset() {
        return this._offset;
    }
    set offset(offset) {
        const parsedType = getParsedType(offset);
        if (parsedType !== ParsedType.object) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.object],
                received: parsedType,
            });
        }
        const parsedOffset = hasKeys(offset, [
            ["hour", "number"],
            ["minute", "number"],
            ["direction", "string"],
        ]);
        if (!parsedOffset.success) {
            switch (true) {
                case parsedOffset.otherKeys.length > 0:
                    throwPgTPError({
                        code: "unrecognized_keys",
                        keys: parsedOffset.otherKeys,
                    });
                    break;
                case parsedOffset.missingKeys.length > 0:
                    throwPgTPError({
                        code: "missing_keys",
                        keys: parsedOffset.missingKeys,
                    });
                    break;
                case parsedOffset.invalidKeys.length > 0:
                    throwPgTPError({
                        code: "invalid_key_type",
                        ...parsedOffset.invalidKeys[0],
                    });
                    break;
                /* c8 ignore next 9 */
                // Assert never
            }
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.object],
                received: parsedType,
            });
        }
        // Validate the offset
        if (parsedOffset.obj.direction !== OffsetDirection.minus && parsedOffset.obj.direction !== OffsetDirection.plus) {
            throwPgTPError({
                code: "invalid_string",
                expected: [OffsetDirection.minus, OffsetDirection.plus],
                received: parsedOffset.obj.direction,
            });
        }
        if (parsedOffset.obj.hour % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
            });
        }
        if (parsedOffset.obj.hour < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
            });
        }
        if (parsedOffset.obj.hour > 23) {
            throwPgTPError({
                code: "too_big",
                maximum: 23,
                type: "number",
                inclusive: true,
            });
        }
        if (parsedOffset.obj.minute % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
            });
        }
        if (parsedOffset.obj.minute < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
            });
        }
        if (parsedOffset.obj.minute > 59) {
            throwPgTPError({
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
            });
        }
        this._offset = parsedOffset.obj;
    }
    get value() {
        return this.toNumber();
    }
    set value(time) {
        const parsed = TimeTZ.safeFrom(time);
        if (parsed.success) {
            this._hour = parsed.data.hour;
            this._minute = parsed.data.minute;
            this._second = parsed.data.second;
            this._offset = parsed.data.offset;
        }
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(time) {
        const parsed = TimeTZ.safeFrom(time);
        if (parsed.success) {
            this._hour = parsed.data.hour;
            this._minute = parsed.data.minute;
            this._second = parsed.data.second;
            this._offset = parsed.data.offset;
        }
        else
            throw parsed.error;
    }
}
export { TimeTZ };
//# sourceMappingURL=TimeTZ.js.map