import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { throwPgTPError } from "../../util/throwPgTPError.js";
class CircleConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        const [argument, ...otherArguments] = context.data, allowedTypes = [ParsedType.number, ParsedType.string, ParsedType.object], parsedType = getParsedType(argument);
        if (parsedType !== ParsedType.number && context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                });
            return INVALID;
        }
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case "string":
                return this._parseString(context, argument);
            case "number":
                return this._parseNumber(context, argument, otherArguments);
            default:
                return this._parseObject(context, argument);
        }
    }
    _parseString(context, argument) {
        // Remove all whitespace
        argument = argument.replaceAll(/\s/g, "");
        if (/^[(<]?\(?(?:-?\d+(\.\d+)?|NaN),(?:-?\d+(\.\d+)?|NaN)\)?,(?:\d+(\.\d+)?|NaN)[)>]?$/.test(argument)) {
            if ((argument.startsWith("<") && !argument.endsWith(">")) ||
                (argument.endsWith(">") && !argument.startsWith("<")) ||
                (argument.startsWith("((") && !argument.endsWith(")")) ||
                (argument.endsWith(")") && !argument.startsWith("(("))) {
                this.setIssueForContext(context, {
                    code: "invalid_string",
                    received: argument,
                    expected: "LIKE <(x,y),radius>",
                });
                return INVALID;
            }
            if (argument.startsWith("<(") || argument.startsWith("(("))
                argument = argument.slice(2);
            if (argument.endsWith(">") || argument.endsWith(")"))
                argument = argument.slice(0, -1);
            const [x, y, radius] = argument.split(",").map(c => Number.parseFloat(c));
            return OK(new CircleClass(x, y, radius));
        }
        this.setIssueForContext(context, {
            code: "invalid_string",
            received: argument,
            expected: "LIKE <(x,y),radius>",
        });
        return INVALID;
    }
    _parseNumber(context, argument, otherArguments) {
        const totalLength = otherArguments.length + 1;
        if (totalLength !== 3) {
            this.setIssueForContext(context, totalLength > 3
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 3,
                    exact: true,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 3,
                    exact: true,
                });
            return INVALID;
        }
        for (const otherArgument of otherArguments) {
            const allowedTypes = [ParsedType.number], parsedType = getParsedType(otherArgument);
            if (!isOneOf(allowedTypes, parsedType)) {
                this.setIssueForContext(context, {
                    code: "invalid_type",
                    expected: allowedTypes,
                    received: parsedType,
                });
                return INVALID;
            }
        }
        const [x, y, radius] = [argument, ...otherArguments];
        return OK(new CircleClass(x, y, radius));
    }
    _parseObject(context, argument) {
        if (this.isCircle(argument))
            return OK(new CircleClass(argument.x, argument.y, argument.radius));
        const parsedObject = hasKeys(argument, [
            ["x", "number"],
            ["y", "number"],
            ["radius", "number"],
        ]);
        if (parsedObject.success)
            return OK(new CircleClass(parsedObject.obj.x, parsedObject.obj.y, parsedObject.obj.radius));
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    isCircle(object) {
        return object instanceof CircleClass;
    }
}
const Circle = new CircleConstructorClass();
class CircleClass extends PgTPBase {
    constructor(_x, _y, _radius) {
        super();
        this._x = _x;
        this._y = _y;
        this._radius = _radius;
    }
    _equals(context) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = Circle.safeFrom(...context.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(context, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return `<(${this._x},${this._y}),${this._radius}>`;
    }
    toJSON() {
        return {
            x: this._x,
            y: this._y,
            radius: this._radius,
        };
    }
    get x() {
        return this._x;
    }
    set x(x) {
        const parsedType = getParsedType(x);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        this._x = x;
    }
    get y() {
        return this._y;
    }
    set y(y) {
        const parsedType = getParsedType(y);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        this._y = y;
    }
    get radius() {
        return this._radius;
    }
    set radius(radius) {
        const parsedType = getParsedType(radius);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        this._radius = radius;
    }
    get value() {
        return this.toString();
    }
    set value(circle) {
        const parsed = Circle.safeFrom(circle);
        if (parsed.success) {
            this._x = parsed.data.x;
            this._y = parsed.data.y;
            this._radius = parsed.data.radius;
        }
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(circle) {
        const parsed = Circle.safeFrom(circle);
        if (parsed.success) {
            this._x = parsed.data.x;
            this._y = parsed.data.y;
            this._radius = parsed.data.radius;
        }
        else
            throw parsed.error;
    }
}
export { Circle };
//# sourceMappingURL=Circle.js.map