import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { throwPgTPError } from "../../util/throwPgTPError.js";
import { Point } from "./Point.js";
class PolygonConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        const [argument, ...otherArguments] = context.data, allowedTypes = [ParsedType.string, ParsedType.object, ParsedType.array], parsedType = getParsedType(argument);
        if (parsedType !== ParsedType.object && context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                });
            return INVALID;
        }
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case "string":
                return this._parseString(context, argument);
            case "array":
                return this._parseArray(context, argument);
            default:
                return this._parseObject(context, argument, otherArguments);
        }
    }
    _parseString(context, argument) {
        // Remove all whitespace
        argument = argument.replaceAll(/\s/g, "");
        if (/^\(?\((?:-?\d+(\.\d+)?|NaN),(?:-?\d+(\.\d+)?|NaN)\)(,\((?:-?\d+(\.\d+)?|NaN),(?:-?\d+(\.\d+)?|NaN)\))*\)?$/.test(argument)) {
            if (argument.startsWith("((") && argument.endsWith("))"))
                argument = argument.slice(1, -1);
            const points = argument
                .split("),(")
                .join("), (")
                .split(", ")
                .map(p => Point.safeFrom(p)), invalidPoint = points.find(p => !p.success);
            /* c8 ignore start */
            if (invalidPoint?.success === false) {
                this.setIssueForContext(context, invalidPoint.error.issue);
                return INVALID;
            }
            /* c8 ignore stop */
            return OK(new PolygonClass(
            //@ts-expect-error - They are all valid at this point
            points.map(range => range.data)));
        }
        this.setIssueForContext(context, {
            code: "invalid_string",
            received: argument,
            expected: "LIKE ((x,y),...)",
        });
        return INVALID;
    }
    _parseArray(context, argument) {
        if (argument.length === 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                type: "array",
                minimum: 1,
                inclusive: true,
            });
            return INVALID;
        }
        const points = argument.map(point => Point.safeFrom(point)), invalidPoint = points.find(point => !point.success);
        if (invalidPoint?.success === false) {
            this.setIssueForContext(context, invalidPoint.error.issue);
            return INVALID;
        }
        return OK(new PolygonClass(
        //@ts-expect-error - They are all valid at this point
        points.map(range => range.data)));
    }
    _parseObject(context, argument, otherArguments) {
        // Input should be [Polygon]
        if (Polygon.isPolygon(argument)) {
            if (otherArguments.length > 0) {
                this.setIssueForContext(context, {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                });
                return INVALID;
            }
            return OK(new PolygonClass(argument.points));
        }
        // Input should be [Point, ...]
        if (Point.isPoint(argument)) {
            const points = otherArguments.map(point => Point.safeFrom(point)), invalidPoint = points.find(point => !point.success);
            if (invalidPoint?.success === false) {
                this.setIssueForContext(context, invalidPoint.error.issue);
                return INVALID;
            }
            return OK(new PolygonClass(
            //@ts-expect-error - They are all valid at this point
            [argument, ...points.map(range => range.data)]));
        }
        // Input should be [PolygonObject | RawPolygonObject]
        if (otherArguments.length > 0) {
            this.setIssueForContext(context, {
                code: "too_big",
                type: "arguments",
                maximum: 1,
                exact: true,
            });
            return INVALID;
        }
        const parsedObject = hasKeys(argument, [["points", "array"]]);
        if (!parsedObject.success) {
            switch (true) {
                case parsedObject.otherKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "unrecognized_keys",
                        keys: parsedObject.otherKeys,
                    });
                    break;
                case parsedObject.missingKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "missing_keys",
                        keys: parsedObject.missingKeys,
                    });
                    break;
                case parsedObject.invalidKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "invalid_key_type",
                        ...parsedObject.invalidKeys[0],
                    });
                    break;
            }
            return INVALID;
        }
        const { points } = parsedObject.obj, parsedPoints = points.map(point => Point.safeFrom(point)), invalidPoint = parsedPoints.find(point => !point.success);
        if (invalidPoint?.success === false) {
            this.setIssueForContext(context, invalidPoint.error.issue);
            return INVALID;
        }
        if (parsedPoints.length === 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                type: "array",
                minimum: 1,
                inclusive: true,
            });
            return INVALID;
        }
        return OK(new PolygonClass(
        //@ts-expect-error - They are all valid at this point
        parsedPoints.map(range => range.data)));
    }
    isPolygon(object) {
        return object instanceof PolygonClass;
    }
}
const Polygon = new PolygonConstructorClass();
class PolygonClass extends PgTPBase {
    constructor(_points) {
        super();
        this._points = _points;
    }
    _equals(context) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = Polygon.safeFrom(...context.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(context, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return `(${this._points.map(p => p.toString()).join(",")})`;
    }
    toJSON() {
        return {
            points: this._points.map(p => p.toJSON()),
        };
    }
    get points() {
        return this._points;
    }
    set points(points) {
        const parsedType = getParsedType(points);
        if (parsedType !== "array") {
            throwPgTPError({
                code: "invalid_type",
                expected: "array",
                received: parsedType,
            });
        }
        if (points.length === 0) {
            throwPgTPError({
                code: "too_small",
                type: "array",
                minimum: 1,
                inclusive: true,
            });
        }
        const finalPoints = points.map(point => Point.safeFrom(point)), invalidPoint = finalPoints.find(point => !point.success);
        if (invalidPoint?.success === false)
            throwPgTPError(invalidPoint.error.issue);
        //@ts-expect-error - They are all valid at this point
        this._points = finalPoints.map(point => point.data);
    }
    get value() {
        return this.toString();
    }
    set value(polygon) {
        const parsed = Polygon.safeFrom(polygon);
        if (parsed.success)
            this._points = parsed.data.points;
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(polygon) {
        const parsed = Polygon.safeFrom(polygon);
        if (parsed.success)
            this._points = parsed.data.points;
        else
            throw parsed.error;
    }
}
export { Polygon };
//# sourceMappingURL=Polygon.js.map