import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "./PgTPBase.js";
import { PgTPConstructorBase } from "./PgTPConstructorBase.js";
import { throwPgTPError } from "./throwPgTPError.js";
const getMultiRange = (object, isObjectFunction, identifier) => {
    const Object = object;
    /* eslint-disable brace-style*/
    class MultiRangeConstructorClass extends PgTPConstructorBase {
        /* eslint-enable brace-style*/
        constructor() {
            super();
        }
        _parse(context) {
            const [argument, ...otherArguments] = context.data, allowedTypes = [ParsedType.string, ParsedType.object, ParsedType.array], parsedType = getParsedType(argument);
            if (parsedType !== ParsedType.object && context.data.length !== 1) {
                this.setIssueForContext(context, context.data.length > 1
                    ? {
                        code: "too_big",
                        type: "arguments",
                        maximum: 1,
                        exact: true,
                    }
                    : {
                        code: "too_small",
                        type: "arguments",
                        minimum: 1,
                        exact: true,
                    });
                return INVALID;
            }
            if (!isOneOf(allowedTypes, parsedType)) {
                this.setIssueForContext(context, {
                    code: "invalid_type",
                    expected: allowedTypes,
                    received: parsedType,
                });
                return INVALID;
            }
            switch (parsedType) {
                case "string":
                    return this._parseString(context, argument);
                case "array":
                    return this._parseArray(context, argument);
                default:
                    return this._parseObject(context, argument, otherArguments);
            }
        }
        _parseString(context, argument) {
            // If the 0 index is set then that automatically means -1 is set
            const [begin, end] = [argument.at(0), argument.at(-1)];
            if (!begin || begin !== "{") {
                this.setIssueForContext(context, {
                    code: "invalid_string",
                    expected: "{",
                    received: begin || "",
                });
                return INVALID;
            }
            if (end !== "}") {
                this.setIssueForContext(context, {
                    code: "invalid_string",
                    expected: "}",
                    received: end,
                });
                return INVALID;
            }
            const halfRanges = argument.slice(1, -1).split(",").filter(Boolean);
            // If halfRanges is an odd number then it's invalid
            if (halfRanges.length % 2 !== 0) {
                this.setIssueForContext(context, {
                    code: "invalid_string",
                    expected: "LIKE {[<DataType>,<DataType>),...}",
                    received: argument,
                });
                return INVALID;
            }
            if (halfRanges.length === 0)
                return OK(new MultiRangeClass([]));
            /**
             * Half ranges splits "{[1,3),[11,13),[21,23)}"
             * to: ["[1", "3)", "[11", "13)", "[21", "23)"]
             * We need to combine the even and odd indexes to get the full ranges
             * ["[1,3)", "[11,13)", "[21,23)"]
             */
            const halfRangesEven = halfRanges.filter((_, index) => index % 2 === 0), halfRangesOdd = halfRanges.filter((_, index) => index % 2 === 1), ranges = halfRangesEven.map((range, index) => `${range},${halfRangesOdd[index]}`).map(range => Object.safeFrom(range)), invalidRange = ranges.find(range => !range.success);
            if (invalidRange?.success === false) {
                this.setIssueForContext(context, invalidRange.error.issue);
                return INVALID;
            }
            //@ts-expect-error - They are all valid at this point
            return OK(new MultiRangeClass(ranges.map(range => range.data)));
        }
        _parseArray(context, argument) {
            const ranges = argument.map(range => Object.safeFrom(range)), invalidRange = ranges.find(range => !range.success);
            if (invalidRange?.success === false) {
                this.setIssueForContext(context, invalidRange.error.issue);
                return INVALID;
            }
            //@ts-expect-error - They are all valid at this point
            return OK(new MultiRangeClass(ranges.map(range => range.data)));
        }
        _parseObject(context, argument, otherArguments) {
            //Input should be [MultiRange<DataType, DataTypeObject>]
            if (MultiRange.isMultiRange(argument)) {
                if (otherArguments.length > 0) {
                    this.setIssueForContext(context, {
                        code: "too_big",
                        type: "arguments",
                        maximum: 1,
                        exact: true,
                    });
                    return INVALID;
                }
                return OK(new MultiRangeClass(argument.ranges));
            }
            //Input should be [Range<DataType, DataTypeObject>, Range<DataType, DataTypeObject>, ...]
            if (isObjectFunction(argument)) {
                if (otherArguments.length === 0)
                    return OK(new MultiRangeClass([argument]));
                const ranges = [argument, ...otherArguments].map(range => Object.safeFrom(range)), invalidRange = ranges.find(range => !range.success);
                if (invalidRange?.success === false) {
                    this.setIssueForContext(context, invalidRange.error.issue);
                    return INVALID;
                }
                //@ts-expect-error - They are all valid at this point
                return OK(new MultiRangeClass(ranges.map(range => range.data)));
            }
            //Input should be [MultiRangeObject<DataType, DataTypeObject> | RawMultiRangeObject<DataTypeObject>]
            if (otherArguments.length > 0) {
                this.setIssueForContext(context, {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                });
                return INVALID;
            }
            const parsedObject = hasKeys(argument, [["ranges", "array"]]);
            if (!parsedObject.success) {
                switch (true) {
                    case parsedObject.otherKeys.length > 0:
                        this.setIssueForContext(context, {
                            code: "unrecognized_keys",
                            keys: parsedObject.otherKeys,
                        });
                        break;
                    case parsedObject.missingKeys.length > 0:
                        this.setIssueForContext(context, {
                            code: "missing_keys",
                            keys: parsedObject.missingKeys,
                        });
                        break;
                    case parsedObject.invalidKeys.length > 0:
                        this.setIssueForContext(context, {
                            code: "invalid_key_type",
                            ...parsedObject.invalidKeys[0],
                        });
                        break;
                }
                return INVALID;
            }
            return this._parseArray(context, parsedObject.obj.ranges);
        }
        isMultiRange(object) {
            //@ts-expect-error - This is a hack to get around the fact that the value is private
            return object instanceof MultiRangeClass && object._identifier === identifier;
        }
    }
    const MultiRange = new MultiRangeConstructorClass();
    class MultiRangeClass extends PgTPBase {
        constructor(_ranges) {
            super();
            this._ranges = _ranges;
            this._identifier = identifier;
        }
        _equals(input) {
            //@ts-expect-error - _equals receives the same context as _parse
            const parsed = MultiRange.safeFrom(...input.data);
            if (parsed.success) {
                return OK({
                    equals: parsed.data.toString() === this.toString(),
                    data: parsed.data,
                });
            }
            this.setIssueForContext(input, parsed.error.issue);
            return INVALID;
        }
        toString() {
            return `{${this._ranges.map(r => r.toString()).join(",")}}`;
        }
        toJSON() {
            return {
                ranges: this._ranges.map(r => r.toJSON()),
            };
        }
        get ranges() {
            return this._ranges;
        }
        set ranges(ranges) {
            if (!Array.isArray(ranges)) {
                throwPgTPError({
                    code: "invalid_type",
                    expected: "array",
                    received: typeof ranges,
                });
            }
            const values = ranges.map(v => Object.safeFrom(v)), invalidValue = values.find(v => !v.success);
            if (invalidValue?.success === false)
                throwPgTPError(invalidValue.error.issue);
            //@ts-expect-error - They are all valid at this point
            this._ranges = values.map(v => v.data);
        }
        get value() {
            return this.toString();
        }
        set value(range) {
            const parsed = MultiRange.safeFrom(range);
            if (parsed.success)
                this._ranges = parsed.data.ranges;
            else
                throw parsed.error;
        }
        get postgres() {
            return this.toString();
        }
        set postgres(range) {
            const parsed = MultiRange.safeFrom(range);
            if (parsed.success)
                this._ranges = parsed.data.ranges;
            else
                throw parsed.error;
        }
    }
    return MultiRange;
};
export { getMultiRange };
//# sourceMappingURL=MultiRange.js.map